﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Views;
using HeuristicLab.Hive.Experiment.Properties;

namespace HeuristicLab.Hive.Experiment.Views {
  /// <summary>
  /// The base class for visual representations of items.
  /// </summary>
  [View("Hive Client View")]
  [Content(typeof(HiveClient), true)]
  public sealed partial class HiveClientView : ItemView {
    public new HiveClient Content {
      get { return (HiveClient)base.Content; }
      set { base.Content = value; }
    }

    /// <summary>
    /// Initializes a new instance of <see cref="ItemBaseView"/>.
    /// </summary>
    public HiveClientView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        this.hiveExperimentListView.Content = null;
      } else {
        this.hiveExperimentListView.Content = Content.HiveExperiments;
      }
    }

    private void updateExperimentsButton_Click(object sender, EventArgs e) {
      ProgressView progressView = new ProgressView(this, "Loading Experiments from server...", true);

      MethodInvoker invoker = new MethodInvoker(Content.UpdateExperimentList);
      invoker.BeginInvoke((ar) => {
        try {
          invoker.EndInvoke(ar);
        }
        finally {
          progressView.Finish();
        }
      }, null);
    }
  }
}
