﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using HeuristicLab.MainForm.WindowsForms;
using System.Windows.Forms;
using System.Diagnostics;

namespace HeuristicLab.Hive.Experiment.Views {
  public partial class ProgressView : HeuristicLab.MainForm.WindowsForms.View {
    private ContentView parentView = null;
    private bool simulateProgress = false;

    public bool CancelEnabled {
      get {
        return cancelButton.Visible;
      }
      set {
        if (InvokeRequired) {
          Invoke(new Action<bool>((ce) => { CancelEnabled = ce; }), value);
        } else {
          cancelButton.Visible = value;
        }
      }
    }

    private string status;
    public string Status {
      get { return status; }
      set {
        if (InvokeRequired) {
          Invoke(new Action<string>((s) => { Status = s;}), value);
        } else {
          status = value;
          statusLabel.Text = status;
        }
        
      }
    }

    public ProgressView(ContentView parentView)
      : this(parentView, "", false, true) {
    }

    public ProgressView(ContentView parentView, string initialStatus)
      : this(parentView, initialStatus, false, true) {
    }

    public ProgressView(ContentView parentView, string initialStatus, bool simulateProgress)
      : this(parentView, initialStatus, simulateProgress, true) {
    }

    /// <param name="parentView">This is the View which will be locked if lockParentView is true</param>
    /// <param name="initialStatus">Status message</param>
    /// <param name="simulateProgress">If true, the progress will be simulated by a timer which addes some progress every second</param>
    /// <param name="lockParentView">if true parentView will be locked</param>
    public ProgressView(ContentView parentView, string initialStatus, bool simulateProgress, bool lockParentView) {
      InitializeComponent();
      this.parentView = parentView;
      this.CancelEnabled = false;
      this.Status = initialStatus;
      this.simulateProgress = simulateProgress;

      if (simulateProgress) {
        Timer progressSimulator = new Timer();
        progressSimulator.Interval = 2000;
        progressSimulator.Tick += new EventHandler(progressSimulator_Tick);
        progressSimulator.Start();
      }

      this.Left = (parentView.ClientRectangle.Width / 2) - (this.Width / 2);
      this.Top = (parentView.ClientRectangle.Height / 2) - (this.Height / 2);
      this.Anchor = AnchorStyles.Left | AnchorStyles.Top;

      if (lockParentView)
        LockBackground();

      parentView.Controls.Add(this);
      this.BringToFront();
    }

    private void progressSimulator_Tick(object sender, EventArgs e) {
      progressBar.Value = Math.Min(progressBar.Value + (progressBar.Maximum - progressBar.Value) / 10 + 1, progressBar.Maximum - 1);
    }

    private void LockBackground() {
      this.parentView.Locked = true;
      foreach (Control c in this.parentView.Controls) {
        c.Enabled = false;
      }
      this.Enabled = true;
      this.ReadOnly = false;
    }

    public void SetProgress(double progress) {
      if (InvokeRequired) {
        Invoke(new Action<double>(SetProgress), progress);
      } else {
        this.progressBar.Value = Math.Max(this.progressBar.Value, (int)(progress * progressBar.Maximum));
      }
    }

    public void Finish() {
      if (InvokeRequired) {
        Invoke(new Action(Finish));
      } else {
        progressBar.Value = progressBar.Maximum;
        parentView.Locked = false;
        foreach (Control c in this.parentView.Controls) {
          c.Enabled = true;
        }
        parentView.Controls.Remove(this);
        this.Dispose();
      }
    }

    private void cancelButton_Click(object sender, EventArgs e) {
      Finish();
    }

    public event EventHandler Canceled;
    protected virtual void OnCanceled() {
      var handler = Canceled;
      if (handler != null) Canceled(this, EventArgs.Empty);
    }
  }
}
