﻿using System;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;

namespace HeuristicLab.Hive.JobBase {
  public interface IJob : INamedItem {
    /// <summary>
    /// This ID is needed to identify the object when it returns from hive
    /// </summary>
    Guid InternalId { get; }

    ExecutionState ExecutionState { get; }
    IEnumerable<IJob> ChildJobs { get; }
    double Progress { get; }
    ILog Log { get; }

    /// <summary>
    /// indicates wether it is possible to create childjobs from this job
    /// </summary>
    bool IsParallelizable { get; }

    /// <summary>
    /// Configuration to indicate if this job should create child-jobs which will be executed in hive
    /// Cannot be set true if IsParallelizable is false
    /// </summary>
    bool ComputeInParallel { get; set; }

    /// <summary>
    /// Gets or sets the cores that will be needed by this job
    /// </summary>
    int CoresNeeded { get; set; }

    /// <summary>
    /// Gets or sets the memory that will be consumed by this job
    /// </summary>
    int MemoryNeeded { get; set; }
    
    void Run();
    void Prepare();
    void Start();
    void Stop();
    void Resume(IEnumerable<IJob> childJobs);

    event EventHandler JobStopped;
    event EventHandler JobFailed;

    /// <summary>
    /// When this event occurs the job wants to sleep until all his child jobs are finished
    /// </summary>
    event EventHandler WaitForChildJobs;

    /// <summary>
    /// This event will be thrown when the job wants to have a child-job to be computed
    /// </summary>
    event EventHandler<Common.EventArgs<IJob>> NewChildJob;
  }
}
