﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.DataAccess;
using System.Threading;
using HeuristicLab.DataAccess.Interfaces;
using System.Transactions;
using HeuristicLab.Hive.Contracts;

namespace HeuristicLab.Hive.Server.Core {
  internal class DefaultScheduler : IScheduler {

    private static object jobLock = new object();

    #region IScheduler Members

    public DefaultScheduler() { }

    public bool ExistsJobForSlave(HeartBeatData hbData) {
      return 
        DaoLocator.JobDao.FindJobsWithFinishedChilds(hbData.SlaveId).Count() > 0 || 
        DaoLocator.JobDao.FindFittingJobs(JobState.Offline, hbData.FreeCores, hbData.FreeMemory, hbData.SlaveId).Count() > 0;
    }

    public JobDto GetNextJobForSlave(Guid slaveId) {
      lock (jobLock) {
        /// Critical section ///
        JobDto jobToCalculate = null;
        using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = ApplicationConstants.ISOLATION_LEVEL_SCOPE })) {
          SlaveDto slave = DaoLocator.SlaveDao.FindById(slaveId);

          if (slave != null) {
            LinkedList<JobDto> finishedJobsForSlave = new LinkedList<JobDto>(DaoLocator.JobDao.FindJobsWithFinishedChilds(slave.Id));
            if (finishedJobsForSlave != null && finishedJobsForSlave.Count > 0) {
              jobToCalculate = finishedJobsForSlave.First.Value;
              UpdateJobAsCalculating(jobToCalculate, slave);
            }

            LinkedList<JobDto> offlineJobsForSlave = new LinkedList<JobDto>(DaoLocator.JobDao.FindFittingJobs(JobState.Offline, slave.NrOfFreeCores, slave.FreeMemory, slave.Id));
            if (offlineJobsForSlave != null && offlineJobsForSlave.Count > 0) {
              jobToCalculate = offlineJobsForSlave.First.Value;
              UpdateJobAsCalculating(jobToCalculate, slave);
            }
          }
          scope.Complete();
        }
        /// End Critical section ///

        return jobToCalculate;
      }
    }

    private static void UpdateJobAsCalculating(JobDto jobToCalculate, SlaveDto slave) {
      jobToCalculate.State = JobState.Calculating;
      jobToCalculate.Slave = slave;
      jobToCalculate.Slave.State = SlaveState.Calculating;
      jobToCalculate.DateCalculated = DateTime.Now;
      DaoLocator.JobDao.AssignSlaveToJob(slave.Id, jobToCalculate.Id);
      DaoLocator.JobDao.Update(jobToCalculate);
      DaoLocator.SlaveDao.Update(jobToCalculate.Slave);
    }

    #endregion
  }
}