﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Windows.Forms;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Hive.Server.Core;

namespace HeuristicLab.Hive.Server {
  [Application("Hive Server", "Server application for the distributed hive engine.", false)]
  public class HeuristicLabHiveServerApplication : ApplicationBase {
    private IDictionary<string, ServiceHost> serviceHosts = new Dictionary<string, ServiceHost>();
    ILifecycleManager lifecycleManager = null;

    public override void Run() {
      try {
        CreateAndOpenServiceHost(typeof(ISlaveFacade), WcfSettings.SlaveServiceName);
        CreateAndOpenServiceHost(typeof(IServerConsoleFacade), WcfSettings.ServerConsoleServiceName);
        CreateAndOpenServiceHost(typeof(IClientFacade), WcfSettings.ClientServiceName);

        lifecycleManager = ServiceLocator.GetLifecycleManager();
        lifecycleManager.Init();

        Form mainForm = new MainForm(GetBaseAddresses());
        Application.Run();
      } finally {
        foreach (ServiceHost host in serviceHosts.Values) {
          if (host.State == CommunicationState.Opened)
            host.Close();
        }
        if (lifecycleManager != null)
          lifecycleManager.Shutdown();
      }
    }

    private ServiceHost CreateAndOpenServiceHost(Type serviceInterfaceType, string serviceName) {
      try {
        ServiceHost host = new ServiceHost(ApplicationManager.Manager.GetTypes(serviceInterfaceType).First());
        host.Open();
        serviceHosts.Add(serviceName, host);
        return host;
      }
      catch (AddressAccessDeniedException ex) {
        throw new Exception("Unable to start WCF-Services due to missing rights. Run the following command as administrator: \"netsh http add urlacl url=http://+:" + WcfSettings.DefaultPort + "/ user=MYMACHINE\\UserName\". See inner exception for more details.", ex);
      }
    }

    private IEnumerable<Uri> GetBaseAddresses() {
      List<Uri> list = new List<Uri>();
      foreach (KeyValuePair<string, ServiceHost> host in serviceHosts) {
        foreach (var addr in host.Value.BaseAddresses) {
          list.Add(addr);
        }
      }
      return list;
    }
  }
}
